/******************************************************************************
Beispiel:      simpleClock

Beschreibung:  Dieses Beispiel soll anhand einer Uhr die Auswertung der Tasten
               auf dem I2C-LCD-Modul verdeutlichen. Nach dem Einschalten wird
               ist zunchst die Uhrzeit einzustellen. Im Display erscheint die
               Anzeige "ST 0" fr die Einstellung der Stunden. Mit Hilfe der 
               Tasten 1 und 2 kann der Wert verndert werden. Durch Bettigen
               der Taste 3 wird zur Einstellung der Minuten gewechselt. Nachdem
               wiederum mit den Tasten 1 und 2 der Wert eingestellt wurde,
               beginnt die Uhr nach Drcken der Taste 3 zu laufen.

Hinweis:  Der Tasteninterrupt des I2C-LCD-Moduls kann auf dem Raspberry Adapter
          nicht verwendet werden.
          
Achtung:  Dieses Beispiel soll im Wesentlichen die Tastenauswertung ber
          das I2C-LCD-Modul verdeutlichen. Fr das Sekundensignal der Uhr wird
          die delay-Funktion verwendet. Dieses ist zum einen sehr ungenau und
          zum anderen sehr ineffizient, jedoch htte eine Verwendung der Timer
          das Beispiel unntig vergrert.
******************************************************************************/

#include "arduPi.h"
#include "TwoWireLCD.h"

int main (){
  setup();
  while(1){
    loop();
  }
  return (0);
}

/******************************************************************************
globale Variablen
******************************************************************************/
// Texte whrend der Zeiteinstellung (ST: Stunde, MI: Minute)
char text[] = "STMI";

// Uhrzeit
unsigned char second = 0;
unsigned char minute = 0;
unsigned char hour = 0;

// aktueller Modus der Anzeige
enum modes
{
    CLOCK_RUN = 0,
    SETTING_MINUTE,
    SETTING_HOUR
};
unsigned char actMode = SETTING_HOUR;

// Zustand der Tasten
unsigned char keys;

/******************************************************************************
Funktionen
******************************************************************************/
void setup()
{  
  // der Interrupt-Pin des I2C-LCD-Moduls liegt auf dem Pin A0 des
  // Arduino-Boards und dieser ist auf dem RPI-AA1 nicht verfgbar,
  // deswegen werden die Tasten zyklisch abgefragt.

  // Initialisierung des I2C und des I2C-LCD-Moduls
  Wire.begin();
  LCD.begin();

  // den Interrupt des I2C-LCD-Moduls zurcksetzen und anschlieend
  // aktiveren.
  LCD.resetKeyInterrupt();
  LCD.enableKeyInterrupt();

  // Dauerschleife bis die Stunden und Minuten eingestellt wurden
  while(actMode)
  {
    // Auswertung der Tasten
    if( LCD.resetKeyInterrupt() )
    {
      //Interrupt Flag gesetzt
       keys = LCD.readKeys();  
    }
    else
    {
        keys = NO_KEY;
    }
    // Taste 1 wurde gedrckt:
    // je nach aktuellem Zustand die Stunden bzw. Minuten hochzhlen
    if((keys & KEY_1) == 0)
    {
      if(actMode == SETTING_HOUR)
      {
        if(hour < 23)
          hour++;
      }
      else if(actMode == SETTING_MINUTE)
      {
        if(minute < 59)
          minute++;
      }  
    }
    
    // Taste 2 wurde gedrckt:
    // je nach aktuellem Zustand die Stunden bzw. Minuten herunterzhlen
    if((keys & KEY_2) == 0)
    {
      if(actMode == 2)
      {
        if(hour > 0)
          hour--;
      }
      else if(actMode == SETTING_HOUR)
      {
        if(minute > 0)
          minute--;
      } 
    }
    
    // Taste 3 wurde gedrckt:
    // in den nchsten Zustand wechseln
    if((keys & KEY_3) == 0)
    {
      --actMode;
    }

      // Ausgabe zyklisch aktualisieren
    if(actMode == SETTING_HOUR)
    {
      LCD.print(&text[0], 2);
      LCD.print(hour, NO_LEADING_ZEROS);
      LCD.update();
    }
    else if(actMode == SETTING_MINUTE)
    {
      LCD.print(&text[2], 2);
      LCD.print(minute, NO_LEADING_ZEROS);
      LCD.update();     
    }
  }
}

void loop()
{
  // Stunden und Minuten anzeigen (Anzeige der Sekunde durch Blinken des Doppelpunkts)
  LCD.clearSegments();
  LCD.print(hour*100 + minute, LEADING_ZEROS);
  LCD.showColon(second & 0x01);
  LCD.update();
  
  // eine Sekunde warten (!!! siehe Beschreibung oben !!!)
  delay(1000); 
  
  second++;
  
  // berlufe der Uhrzeit auswerten
  if(second > 59)
  {
    second = 0;
    minute++;
    
    if(minute > 59)
    {
      minute = 0;
      hour++;
      
      if(hour > 23)
        hour = 0;
    }
  }
}
