#ifndef PWMPi_h
#define PWMPi_h

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <string.h>
#include <stdint.h>

namespace unistd {
  //All functions of unistd.h must be called like this: unistd::the_function()
  #include <unistd.h>
}

#define IOBASE                  0x20000000

#define GPIO_BASE               (IOBASE + 0x200000)
#define PWM_BASE                (IOBASE + 0x20C000)
#define CLK_BASE                (IOBASE + 0x101000)

#define BLOCK_SIZE              4096

#define BCM2835_PWM_GPFSEL0     0x0000 ///< GPIO Function Select 0

typedef enum
{
  BCM2835_PWM_GPIO_FSEL_INPT  = 0b000,   ///< Input
  BCM2835_PWM_GPIO_FSEL_OUTP  = 0b001,   ///< Output
  BCM2835_PWM_GPIO_FSEL_ALT0  = 0b100,   ///< Alternate function 0
  BCM2835_PWM_GPIO_FSEL_ALT1  = 0b101,   ///< Alternate function 1
  BCM2835_PWM_GPIO_FSEL_ALT2  = 0b110,   ///< Alternate function 2
  BCM2835_PWM_GPIO_FSEL_ALT3  = 0b111,   ///< Alternate function 3
  BCM2835_PWM_GPIO_FSEL_ALT4  = 0b011,   ///< Alternate function 4
  BCM2835_PWM_GPIO_FSEL_ALT5  = 0b010,   ///< Alternate function 5
  BCM2835_PWM_GPIO_FSEL_MASK  = 0b111    ///< Function select bits mask
} bcm2835_PWM_FunctionSelect;

// Defines for PWM, word offsets (ie 4 byte multiples)
#define BCM2835_PWM_CONTROL     0
#define BCM2835_PWM_STATUS      1
#define BCM2835_PWM_DMAC        2
#define BCM2835_PWM0_RANGE      4
#define BCM2835_PWM0_DATA       5
#define BCM2835_PWM_FIF1        6
#define BCM2835_PWM1_RANGE      8
#define BCM2835_PWM1_DATA       9

// Defines for PWM Clock, word offsets (ie 4 byte multiples)
#define BCM2835_PWMCLK_CNTL     40
#define BCM2835_PWMCLK_DIV      41
#define BCM2835_PWM_PASSWRD     (0x5A << 24)  ///< Password to enable setting PWM clock

#define BCM2835_PWM1_MS_MODE    0x8000  ///< Run in Mark/Space mode
#define BCM2835_PWM1_USEFIFO    0x2000  ///< Data from FIFO
#define BCM2835_PWM1_REVPOLAR   0x1000  ///< Reverse polarity
#define BCM2835_PWM1_OFFSTATE   0x0800  ///< Ouput Off state
#define BCM2835_PWM1_REPEATFF   0x0400  ///< Repeat last value if FIFO empty
#define BCM2835_PWM1_SERIAL     0x0200  ///< Run in serial mode
#define BCM2835_PWM1_ENABLE     0x0100  ///< Channel Enable

#define BCM2835_PWM0_MS_MODE    0x0080  ///< Run in Mark/Space mode
#define BCM2835_PWM_CLEAR_FIFO  0x0040  ///< Clear FIFO
#define BCM2835_PWM0_USEFIFO    0x0020  ///< Data from FIFO
#define BCM2835_PWM0_REVPOLAR   0x0010  ///< Reverse polarity
#define BCM2835_PWM0_OFFSTATE   0x0008  ///< Ouput Off state
#define BCM2835_PWM0_REPEATFF   0x0004  ///< Repeat last value if FIFO empty
#define BCM2835_PWM0_SERIAL     0x0002  ///< Run in serial mode
#define BCM2835_PWM0_ENABLE     0x0001  ///< Channel Enable

typedef enum
{
  PWM_CLOCK_DIVIDER_32768 = 32768,   ///< 32768 = 585Hz
  PWM_CLOCK_DIVIDER_16384 = 16384,   ///< 16384 = 1171.8Hz
  PWM_CLOCK_DIVIDER_8192  = 8192,    ///< 8192 = 2.34375kHz
  PWM_CLOCK_DIVIDER_4096  = 4096,    ///< 4096 = 4.6875kHz
  PWM_CLOCK_DIVIDER_2048  = 2048,    ///< 2048 = 9.375kHz
  PWM_CLOCK_DIVIDER_1024  = 1024,    ///< 1024 = 18.75kHz
  PWM_CLOCK_DIVIDER_512   = 512,     ///< 512 = 37.5kHz
  PWM_CLOCK_DIVIDER_256   = 256,     ///< 256 = 75kHz
  PWM_CLOCK_DIVIDER_128   = 128,     ///< 128 = 150kHz
  PWM_CLOCK_DIVIDER_64    = 64,      ///< 64 = 300kHz
  PWM_CLOCK_DIVIDER_32    = 32,      ///< 32 = 600.0kHz
  PWM_CLOCK_DIVIDER_16    = 16,      ///< 16 = 1.2MHz
  PWM_CLOCK_DIVIDER_8     = 8,       ///< 8 = 2.4MHz
  PWM_CLOCK_DIVIDER_4     = 4,       ///< 4 = 4.8MHz
  PWM_CLOCK_DIVIDER_2     = 2,       ///< 2 = 9.6MHz, fastest you can get
  PWM_CLOCK_DIVIDER_1     = 1,       ///< 1 = 4.6875kHz, same as divider 4096
} PWMClockDivider;

class PWMPi{
  private:
    void delay(long millis);
    void *PWM_mapmem(const char *msg, size_t size, int fd, off_t off);
    void PWM_unmapmem(void **pmem, size_t size);
    uint32_t bcm2835_PWM_peri_read(volatile uint32_t* paddr);
    uint32_t bcm2835_PWM_peri_read_nb(volatile uint32_t* paddr);
    void bcm2835_PWM_peri_write(volatile uint32_t* paddr, uint32_t value);
    void bcm2835_PWM_peri_write_nb(volatile uint32_t* paddr, uint32_t value);
    void bcm2835_PWM_peri_set_bits(volatile uint32_t* paddr, uint32_t value, uint32_t mask);
    void bcm2835_PWM_gpio_fsel(uint8_t pin, uint8_t mode);
  public:
    PWMPi();
    void begin();
    void close();
    void setClockDivider(uint32_t divider);
    void setMode(uint8_t markspace, uint8_t enabled);
    void setRange(uint32_t range);
    void setData(uint32_t data);
};

extern PWMPi PWM;
#endif