extern "C" 
{
#include "CP240x_reg.h"
#include "CP240x_lcd.h"
#include "lcd.h"
}

#include "arduPi.h"
#include "TwoWireLCD.h"

/**********************************************************
private Definitionen
**********************************************************/
#define I2C_LCD_ADDRESS  (0x76 >> 1)

/**********************************************************
Konstruktor
**********************************************************/
TwoWireLCD::TwoWireLCD()
{
}

/**********************************************************
öffentliche Methoden
**********************************************************/

/*---------------------------------------------------------
Name:           begin

Beschreibung:   initialisiert das I2C-LCD-Modul            
 
Eingänge:       void      

Ausgang:        void
---------------------------------------------------------*/
void TwoWireLCD::begin(void)
{
  unsigned char i = 0;
  
  while(CP240xInitSequence[i] != 0)
  {
    writeRegister(I2C_LCD_ADDRESS, CP240xInitSequence[i+1], CP240xInitSequence[i+2], &CP240xInitSequence[i+3], CP240xInitSequence[i]);
    
    i += (3 + CP240xInitSequence[i]);
  }
}

/*---------------------------------------------------------
Name:           print

Beschreibung:   füllt die Ausgaberesgister für die Anzeige
                eines Wertes zwischen -9999 und 9999 auf
                dem Display.
                (Anzeige erfolgt mit dem Aufruf von update())
 
Eingänge:       signed int value
                darzustellender Wert (Werte größer 9999
                und kleiner -9999 werden auf die maximal
                darstellbaren Werte begrenzt)

                unsigned char showLeadingZeros
                zeigt an, ob führende Nullen dargestellt
                werden soll.

                showLeadingZeros == 0: führende Nullen 
                werden nicht dargestellt

                showLeadingZeros != 0: führende Nullen 
                werden dargestellt

Ausgang:        unsigned char
                Anzahl der dargestellten Zeichen
---------------------------------------------------------*/
unsigned char TwoWireLCD::print(signed int value, unsigned char showLeadingZeros)
{
  char i;
  unsigned int div;
  unsigned char rest;
  
  // ist der Wert kleiner 0?
  // falls ja:  Minuszeichen darstellen und für weitere
  //  Verarbeitung den absoluten Wert der Zahl
  //  berechnen.
  // falls nein:  Minuszeichen ausschalten
  if(value < 0)
  {
    showMinus(1);
    value = -value; 
  }
  else
  {
    showMinus(0); 
  }
  
  // Wert auf 9999 begrenzen, da die Anzeige nicht mehr
  // darstellen kann.
  if(value > 9999)
    value = 9999;
    
  // Berechnung und Darstellung der einzelnen stellen
  div = value / 10;
  rest = value - 10 * div;
  
  for(i = 4; i > 0; i--)
  {
    setSegment(i, rest);
    
    if((showLeadingZeros == 0) && (div == 0) || (i == 1))
      break;
      
    value = div;
    div = value / 10;
    rest = value - 10 * div;
  }
  
  // Anzahl der dargestellten Zeichen zurückgeben
  return (5-i);
}

/*---------------------------------------------------------
Name:           print

Beschreibung:   füllt die Ausgaberesgister für die Anzeige
                eines Wertes zwischen -9999 und 9999 auf
                dem Display, wobei unter Angabe der Nachkomma-
                stellen Festkommazahlen darstellbar sind.
                (Anzeige erfolgt mit dem Aufruf von update())

              Beispiel:
                print(1234, 2) liefert auf dem Display
                die Darstellung 12.34
                print(1, 2) liefert auf dem Display
                die Darstellung 0.01
 
Eingänge:       signed int value
                darzustellender Wert (Werte größer 9999
                und kleiner -9999 werden auf die maximal
                darstellbaren Werte begrenzt)

                unsigned char showLeadingZeros
                zeigt an, ob führende Nullen dargestellt
                werden soll.

                showLeadingZeros == 0: führende Nullen 
                werden nicht dargestellt

                showLeadingZeros != 0: führende Nullen 
                werden dargestellt

                unsigned char decimalPlace
                Nachkommastellen

Ausgang:        void
---------------------------------------------------------*/
void TwoWireLCD::print(signed int value, unsigned char showLeadingZeros, unsigned char decimalPlace)
{
  // Wert ausgeben
  unsigned char showingCharacters = print(value, showLeadingZeros);
  
  // bei Werten mit weniger Stellen als die Anzahl der Nachkomma-
  // stellen mit Nullen auffüllen.
  while(showingCharacters <= decimalPlace)
  {
    setSegment(4-showingCharacters, 0);
    showingCharacters++;
  }
  
  // Punkt darstellen
  showPoints(1<<(3-decimalPlace));
}

/*---------------------------------------------------------
Name:           print

Beschreibung:   füllt die Ausgaberesgister für die Anzeige
                einer Zeichenkette im ASCII-Format 
                auf dem Display.
                (Anzeige erfolgt mit dem Aufruf von update())

                möglich Zeichen sind:
                0,1,2,3,4,5,6,7,8,9,A,B,C,D,E,F,G,H,I,J,K,
                L,M,N,O,P,Q,R,S,T,U,V,W,X,Y,Z,+,-,/,\,*,
                (Leerzeichen)

                - alle anderen ASCII-Zeichen werden als 
                Leerzeichen dargestellt.
                - es wird nicht zwischen Groß- und Klein-
                schreibung unterschieden (a wird als A
                dargestellt)
          
Eingänge:       char* text
                darzustellende Zeichenkette im ASCII-Format

                unsigned char count
                Anzahl der darzustellenden Zeichen
                (größere Werte als 4 Zeichen werden auf 4 begrenzt)

Ausgang:        void
---------------------------------------------------------*/
void TwoWireLCD::print(char* text, unsigned char count)
{
  unsigned char i;
  unsigned char index;
  
  // Begrenzung auf 4 Zeichen
  if(count > 4)
    count = 4;
    
  // Index der Zeichen innerhalb der Lookup-Table ermitteln
  // und darstellen
  for(i = 0; i < 4; i++)
  {
    if(i >= count)
      index = INDEX_SYMBOL_BLANK;
    else if(text[i] >= 'A' && text[i] <= 'Z')
      index = 10 + (text[i] - 'A');
    else if(text[i] >= 'a' && text[i] <= 'z')
      index = 10 + (text[i] - 'a'); 
    else if(text[i] == '+')
      index = INDEX_SYMBOL_PLUS;
    else if(text[i] == '-')
      index = INDEX_SYMBOL_MINUS;
    else if(text[i] == '/')
      index = INDEX_SYMBOL_SLASH;
    else if(text[i] == 0x5C)  //"\"
      index = INDEX_SYMBOL_BACKSLASH;
    else if(text[i] == '*')
      index = INDEX_SYMBOL_ASTERISK;
    else                      // alles andere wird als Leerzeichen dargestellt
      index = INDEX_SYMBOL_BLANK;
      
    setSegment(i+1, index);
  }  
}

/*---------------------------------------------------------
Name:           update

Beschreibung:   zeigt die vorher in die Ausgaberegister 
                geschriebenen Zeichen auf dem Display an.
          
Eingänge:       void

Ausgang:        void
---------------------------------------------------------*/
void TwoWireLCD::update(void)
{
  writeRegister(I2C_LCD_ADDRESS, REGWRITE, CP240X_ULPMEM00, lcdData, 10);
}

/*---------------------------------------------------------
Name:           clearAll

Beschreibung:   löscht alle Zeichen auf dem Display.
                (Übernahme aufs Display erfolgt nach dem Aufruf von update())
            
Eingänge:       void

Ausgang:        void
---------------------------------------------------------*/
void TwoWireLCD::clearAll(void)
{
  unsigned char i;
  for(i = 0; i < 10; i++)
    lcdData[i] = 0;
}

/*---------------------------------------------------------
Name:           clearSegments

Beschreibung:   löscht nur die vier Segmente.
                (Übernahme aufs Display erfolgt nach dem Aufruf von update())
            
Eingänge:       void

Ausgang:        void
---------------------------------------------------------*/
void TwoWireLCD::clearSegments(void)
{
  unsigned char i;
  for(i = 1; i <= 4; i++)
    setSegment(i, INDEX_SYMBOL_BLANK);
}

/*---------------------------------------------------------
Name:           showDegree

Beschreibung:   setzt bzw. löscht die Darstellung des °C-Symbols im Ausgaberegister
                (Übernahme aufs Display erfolgt nach dem Aufruf von update())
              
Eingänge:       unsigned char visible
                zeigt an, ob das °C-Symbol dargestellt
                werden soll.

                visible != 0: °C-Symbol wird angezeigt
                visible == 0: °C-Symbol wird nicht angezeigt

Ausgang:        void
---------------------------------------------------------*/
void TwoWireLCD::showDegree(unsigned char visible)
{
  if(visible)
    lcdData[1] |= (1<<3);
  else
    lcdData[1] &= ~(1<<3);
}

/*---------------------------------------------------------
Name:           showPercent

Beschreibung:   setzt bzw. löscht die Darstellung des %-Symbols im Ausgaberegister
                (Übernahme aufs Display erfolgt nach dem Aufruf von update())
          
Eingänge:       unsigned char visible
                zeigt an, ob das %-Symbol dargestellt
                werden soll.

                visible != 0: %-Symbol wird angezeigt
                visible == 0: %-Symbol wird nicht angezeigt

Ausgang:        void
---------------------------------------------------------*/
void TwoWireLCD::showPercent(unsigned char visible)
{
  if(visible)
    lcdData[1] |= (1<<2);
  else
    lcdData[1] &= ~(1<<2);
}

/*---------------------------------------------------------
Name:           showMicro

Beschreibung:   setzt bzw. löscht die Darstellung des µ-Symbols im Ausgaberegister
                (Übernahme aufs Display erfolgt nach dem Aufruf von update())
            
Eingänge:       unsigned char visible
                zeigt an, ob das µ-Symbol dargestellt
                werden soll.

                visible != 0: µ-Symbol wird angezeigt
                visible == 0: µ-Symbol wird nicht angezeigt

Ausgang:        void
---------------------------------------------------------*/
void TwoWireLCD::showMicro(unsigned char visible)
{
  if(visible)
    lcdData[1] |= (1<<1);
  else
    lcdData[1] &= ~(1<<1);
}

/*---------------------------------------------------------
Name:           showMilli

Beschreibung:   setzt bzw. löscht die Darstellung des m-Symbols im Ausgaberegister
                (Übernahme aufs Display erfolgt nach dem Aufruf von update())
                
Eingänge:       unsigned char visible
                zeigt an, ob das m-Symbol dargestellt
                werden soll.
  
                visible != 0: m-Symbol wird angezeigt
                visible == 0: m-Symbol wird nicht angezeigt

Ausgang:        void
---------------------------------------------------------*/
void TwoWireLCD::showMilli(unsigned char visible)
{
  if(visible)
    lcdData[1] |= (1<<0);
  else
    lcdData[1] &= ~(1<<0);
}

/*---------------------------------------------------------
Name:           showAmpere

Beschreibung:   setzt bzw. löscht die Darstellung des A-Symbols im Ausgaberegister
                (Übernahme aufs Display erfolgt nach dem Aufruf von update())
                
Eingänge:       unsigned char visible
                zeigt an, ob das A-Symbol dargestellt
                werden soll.

                visible != 0: A-Symbol wird angezeigt
                visible == 0: A-Symbol wird nicht angezeigt

Ausgang:        void
---------------------------------------------------------*/
void TwoWireLCD::showAmpere(unsigned char visible)
{
  if(visible)
    lcdData[0] |= (1<<4);
  else
    lcdData[0] &= ~(1<<4);
}

/*---------------------------------------------------------
Name:           showHour

Beschreibung:   setzt bzw. löscht die Darstellung des h-Symbols im Ausgaberegister
                (Übernahme aufs Display erfolgt nach dem Aufruf von update())
                
Eingänge:       unsigned char visible
                zeigt an, ob das h-Symbol dargestellt
                werden soll.

                visible != 0: h-Symbol wird angezeigt
                visible == 0: h-Symbol wird nicht angezeigt

Ausgang:        void
---------------------------------------------------------*/
void TwoWireLCD::showHour(unsigned char visible)
{
  if(visible)
    lcdData[0] |= (1<<5);
  else
    lcdData[0] &= ~(1<<5);
}

/*---------------------------------------------------------
Name:           showOhm

Beschreibung:   setzt bzw. löscht die Darstellung des Ohm-Symbols im Ausgaberegister
                (Übernahme aufs Display erfolgt nach dem Aufruf von update())
              
Eingänge:       unsigned char visible
                zeigt an, ob das Ohm-Symbol dargestellt
                werden soll.

                visible != 0: Ohm-Symbol wird angezeigt
                visible == 0: Ohm-Symbol wird nicht angezeigt

Ausgang:        void
---------------------------------------------------------*/
void TwoWireLCD::showOhm(unsigned char visible)
{
  if(visible)
    lcdData[0] |= (1<<6);
  else
    lcdData[0] &= ~(1<<6);
}

/*---------------------------------------------------------
Name:           showVolt

Beschreibung:   setzt bzw. löscht die Darstellung des V-Symbols im Ausgaberegister
                (Übernahme aufs Display erfolgt nach dem Aufruf von update())
                
Eingänge:       unsigned char visible
                zeigt an, ob das V-Symbol dargestellt
                werden soll.

                visible != 0: V-Symbol wird angezeigt
                visible == 0: V-Symbol wird nicht angezeigt

Ausgang:        void
---------------------------------------------------------*/
void TwoWireLCD::showVolt(unsigned char visible)
{
  if(visible)
    lcdData[0] |= (1<<7);
  else
    lcdData[0] &= ~(1<<7);
}

/*---------------------------------------------------------
Name:           showColon

Beschreibung:   setzt bzw. löscht die Darstellung des :-Symbols im Ausgaberegister
                (Übernahme aufs Display erfolgt nach dem Aufruf von update())
                
Eingänge:       unsigned char visible
                zeigt an, ob das :-Symbol dargestellt
                werden soll.

                visible != 0: :-Symbol wird angezeigt
                visible == 0: :-Symbol wird nicht angezeigt

Ausgang:        void
---------------------------------------------------------*/
void TwoWireLCD::showColon(unsigned char visible)
{
  if(visible)
    lcdData[7] |= (1<<2);
  else
    lcdData[7] &= ~(1<<2);  
}

/*---------------------------------------------------------
Name:           showMinus

Beschreibung:   setzt bzw. löscht die Darstellung des Minus-Symbols im Ausgaberegister
                (Übernahme aufs Display erfolgt nach dem Aufruf von update())
                
Eingänge:       unsigned char visible
                zeigt an, ob das Minus-Symbol dargestellt
                werden soll.

                visible != 0: Minus-Symbol wird angezeigt
                visible == 0: Minus-Symbol wird nicht angezeigt

Ausgang:        void
---------------------------------------------------------*/
void TwoWireLCD::showMinus(unsigned char visible)
{
  if(visible)
    lcdData[1] |= (1<<6);
  else
    lcdData[1] &= ~(1<<6);  
}

/*---------------------------------------------------------
Name:           showBattery

Beschreibung:   setzt bzw. löscht die Darstellung des Batterie-Symbols im Ausgaberegister
                (Übernahme aufs Display erfolgt nach dem Aufruf von update())
                
Eingänge:       unsigned char decision value
                zeigt an, ob das Batterie-Symbol dargestellt
                werden soll.

                visible != 0: Batterie-Symbol wird angezeigt
                visible == 0: Batterie-Symbol wird nicht angezeigt

Ausgang:        void
---------------------------------------------------------*/
void TwoWireLCD::showBattery(unsigned char visible)
{
  if(visible)
    lcdData[1] |= (1<<7);
  else
    lcdData[1] &= ~(1<<7);
}

/*---------------------------------------------------------
Name:           showPoints

Beschreibung:   setzt bzw. löscht die Darstellung der Punkte im Ausgaberegister
                (Übernahme aufs Display erfolgt nach dem Aufruf von update())
                
Eingänge:       unsigned char points
                bitkodierte Darstellung der Punkte

                Bit 0: Punkt zwischen dem Zeichen 1 und 2
                Bit 1: Punkt zwischen dem Zeichen 2 und 3
                Bit 2: Punkt zwischen dem Zeichen 3 und 4

                Beispiel:
                points = 0x03 (= 0b00000011)

                Der Punkt zwischen den Zeichen 1 und 2 und
                der Punkt zwischen den Zeichen 2 und 3
                werden dargestellt.

Ausgang:        void
---------------------------------------------------------*/
void TwoWireLCD::showPoints(unsigned char points)
{
  if(points & POINT_1)
    lcdData[9] |= (1<<2);
  else
    lcdData[9] &= ~(1<<2);
    
  if(points & POINT_2)
    lcdData[7] |= (1<<3);
  else
    lcdData[7] &= ~(1<<3);
    
  if(points & POINT_3)
    lcdData[5] |= (1<<2);
  else
    lcdData[5] &= ~(1<<2);
}


/*---------------------------------------------------------
Name:           showArrows

Beschreibung:   setzt bzw. löscht die Darstellung der Pfeile am rechten Rand
                des Displays im Ausgaberegister
                (Übernahme aufs Display erfolgt nach dem Aufruf von update())
                
Eingänge:       unsigned char arrows
                bitkodierte Darstellung der Pfeile

                Bit 0: 1. Pfeil von oben wird dargestellt
                Bit 1: 2. Pfeil von oben wird dargestellt
                Bit 2: 3. Pfeil von oben wird dargestellt
                Bit 3: 4. Pfeil von oben wird dargestellt

                Beispiel:
                arrows = 0x05 (= 0b00000101)

                Der 2. und der 4. Pfeil von oben werden
                dargestellt.

Ausgang:        void
---------------------------------------------------------*/
void TwoWireLCD::showArrows(unsigned char arrows)
{
  lcdData[0] &= 0xF0;
  lcdData[0] |= (arrows & 0x0F);
}

/*---------------------------------------------------------
Name:           enableKeyInterrupt

Beschreibung:   aktiviert die Interrupt-Leitung des I2C-LCD-Moduls
                für Tasten (bei einem Tastendruck wird ein Interrupt
                ausgelöst).
                
Eingänge:       void

Ausgang:        void
---------------------------------------------------------*/
void TwoWireLCD::enableKeyInterrupt(void)
{
  unsigned char registerValue;
  
  // Maske für P3 festlegen
  registerValue = 0xF0;
  writeRegister(I2C_LCD_ADDRESS, REGWRITE, CP240X_P3MSK, &registerValue, 1);
  
  // Vergleichswerte für P3 festlegen
  registerValue = 0xF0;
  writeRegister(I2C_LCD_ADDRESS, REGWRITE, CP240X_P3MATCH, &registerValue, 1);
  
  // Match Interrupt einschalten
  registerValue = 0x01;
  writeRegister(I2C_LCD_ADDRESS, REGWRITE, CP240X_INT0EN, &registerValue, 1);
}

/*---------------------------------------------------------
Name:           disableKeyInterrupt

Beschreibung:   deaktiviert die Interrupt-Leitung des I2C-LCD-Moduls
                für Tasten (bei einem Tastendruck wird kein Interrupt
                ausgelöst).
                
Eingänge:       void

Ausgang:        void
---------------------------------------------------------*/
void TwoWireLCD::disableKeyInterrupt(void)
{
  unsigned char registerValue;
  
  // Match Interrupt ausschalten
  registerValue = 0x00;
  writeRegister(I2C_LCD_ADDRESS, REGWRITE, CP240X_INT0EN, &registerValue, 1);
}

/*---------------------------------------------------------
Name:           resetKeyInterrupt

Beschreibung:   setzt den Interrupt des I2C-LCD-Moduls nach
                einem eingegangenen Interrupt wieder zurück
                (nach Aufruf dieser Methode ist die Taster-
                auswertung wieder "scharf" geschaltet.)
                
Eingänge:       void

Ausgang:        void
---------------------------------------------------------*/
unsigned char TwoWireLCD::resetKeyInterrupt(void)
{
  unsigned char int0;
  readRegister(I2C_LCD_ADDRESS, REGREAD, CP240X_INT0, 1, &int0);
  return int0;
}

/*---------------------------------------------------------
Name:           readKeys

Beschreibung:   liest den Zustand der Taster ein.
                
Eingänge:       void

Ausgang:        unsigned char
                Zustand der Taster (bitkodiert)

                Achtung: negative Logik

                0: Taster ist gedrückt
                1: Taster ist nicht gedrückt
---------------------------------------------------------*/
unsigned char TwoWireLCD::readKeys(void)
{
  unsigned char keys;
  readRegister(I2C_LCD_ADDRESS, REGREAD, CP240X_P3IN, 1, &keys);   
  return keys;
}

/*---------------------------------------------------------
Name:           setLeds

Beschreibung:   schaltet die LEDs D1 bis D4 ein bzw. aus.
                
Eingänge:       unsigned char ledOutput
                bitkodierter Zustand der LEDs

                Bit 0: LED D1
                Bit 1: LED D2
                Bit 2: LED D3
                Bit 3: LED D4

                Achtung: negative Logik

                0: LED wird eingeschaltet
                1: LED wird ausgeschaltet

Ausgang:        void
---------------------------------------------------------*/
void TwoWireLCD::setLeds(unsigned char ledOutput)
{
  ledOutput = ledOutput | 0xF0;    // obere 4 Bits immer high, da diese als Eingang genutzt werden
  writeRegister(I2C_LCD_ADDRESS, REGWRITE, CP240X_P3OUT, &ledOutput, 1);  
}

/**********************************************************
private Methoden
**********************************************************/
/*---------------------------------------------------------
Name:           writeRegister

Beschreibung:   beschreibt die Register des CP2401 auf dem
                I2C-LCD-Modul
                
Eingänge:       unsigned char address
                  I2C-Adresse des CP2401
                
                unsigned char command
                  Kommunikationbefehl 
                  (einfaches Schreiben, sequentielles Schreiben)
                
                unsigned char regAddress
                  zu beschreibende Registeradresse (Startadresse)

                unsigned char *data
                  zu schreibende Daten

                unsigned char datalength
                  Anzahl der zu sendenden Bytes

Ausgang:        void
---------------------------------------------------------*/
void TwoWireLCD::writeRegister(unsigned char address, unsigned char command, unsigned char regAddress, unsigned char *data, unsigned char datalength)
{
  char temp_data[datalength+3];
  temp_data[0] = command;
  temp_data[1] = 0x00;
  temp_data[2] = regAddress;
  
  for(unsigned char i = 0; i < datalength; i++)
  {
    temp_data[i+3] = *(data+i);
  }
  
  Wire.beginTransmission(address);
  // Wire.write((uint8_t)command);
  // Wire.write((uint8_t)0x00);
  // Wire.write((uint8_t)regAddress);
  // Wire.write((char*)data, datalength);
  Wire.write(&temp_data[0], (datalength+3));
  
  Wire.endTransmission();
}

/*---------------------------------------------------------
Name:           readRegister

Beschreibung:   liest die Register des CP2401 auf dem
                  I2C-LCD-Modul
                  
Eingänge:       unsigned char address
                  I2C-Adresse des CP2401
                
                unsigned char command
                  Kommunikationbefehl 
                  (einfaches Lesen, sequentielles Lesen)
                
                unsigned char reg_address
                  zu lesende Registeradresse (Startadresse)

                unsigned char datalength
                  Anzahl der zu lesenden Bytes

                unsigned char *data
                  Speicher für die gelesenen Daten

Ausgang:        unsigned char
                  Anzahl der gelesenen Bytes
---------------------------------------------------------*/
unsigned char TwoWireLCD::readRegister(unsigned char address, unsigned char command, unsigned char regAddress, unsigned char length, unsigned char *data)
{
  unsigned char readBytes = 0;
  char temp_data[3];
  temp_data[0] = command;
  temp_data[1] = 0x00;
  temp_data[2] = regAddress;
  
  Wire.beginTransmission(address);
  // Wire.write((uint8_t)command);
  // Wire.write((uint8_t)0x00);
  // Wire.write((uint8_t)regAddress);
  Wire.write(&temp_data[0], 3);

  delay(10);
  
  length += 4;
  
  Wire.requestFrom(address, length);
  
  //char read_data[length];
  // Wire.read( read_data );
  // //length -= 4;
  // while(length>4)
  // {
    // data[readBytes] = read_data[length-1];
    // length--;
    // readBytes++;
  // }
  
  delay(10);
  
  Wire.read();
  Wire.read();
  Wire.read();
  Wire.read();
  
  while( Wire.available() )
  {
    data[readBytes] = Wire.read();
    readBytes++;
  }
  
  return readBytes;
}

/*---------------------------------------------------------
Name:           setSegment

Beschreibung:   setzt die Daten im Ausgaberegister für ein
                  ein bestimmtes Segment
                  
Eingänge:       unsigned char segment
                  Segment für das ein neues Zeichen
                  angezeigt werden soll
                
                unsigned char indexLcdSymbol
                  Index des anzuzeigenden Symbols (siehe lcd.h)

Ausgang:        void
---------------------------------------------------------*/
void TwoWireLCD::setSegment(unsigned char segment, unsigned char indexLcdSymbol)
{
  // temporäre Variable mit unterschiedlicher Funktion
  // für Segment 1: Zwischenspeicher für die anzuzeigenden Daten
  // für Segement 2-4:  Zwischenspeicher für die Indizes der neu zu beschreibenden
  //   Einträge im Ausgabespeicher
  unsigned char temp;
  
  indexLcdSymbol *= 2;
  
  // Segment 1 ist ein Sonderfall, da die Register für dieses Segment anders
  // angeordnet sind als bei den Segmenten 2-4
  if(segment == 1)
  {
    temp = lcdSymbols[indexLcdSymbol];
    lcdData[2] = ((temp & 0x0F) << 4) | ((temp & 0xF0) >> 4);
    lcdData[1] &= 0xCF;
    lcdData[1] |= (lcdSymbols[indexLcdSymbol+1] << 4);
    lcdData[3] &= 0xF0;
    lcdData[3] |= (lcdSymbols[indexLcdSymbol+1] >> 4);
  }  
  else
  {
    temp = 12 - 2 * segment;    
    lcdData[temp] = lcdSymbols[indexLcdSymbol];
    lcdData[temp-1] &= 0x0F;
    lcdData[temp-1] |= lcdSymbols[indexLcdSymbol+1] & 0xF0;
    lcdData[temp+1] &= 0xFC;
    lcdData[temp+1] |= lcdSymbols[indexLcdSymbol+1] & 0x03;  
  }
}

/**********************************************************
Vorinstantiiertes Objekt
**********************************************************/
TwoWireLCD LCD = TwoWireLCD();