#include "arduPi.h"
#include "TwoWireRTC.h"

/**********************************************************
private Definitionen
**********************************************************/
#define I2C_RTC_ADDRESS  (0x64 >> 1)

/**********************************************************
Konstruktor
**********************************************************/
TwoWireRTC::TwoWireRTC()
{}

/**********************************************************
ffentliche Methoden
**********************************************************/

/*---------------------------------------------------------
Name:           begin

Beschreibung:   initialisiert das I2C-RTC-Modul            
 
Eingnge:       void      

Ausgang:        void
---------------------------------------------------------*/
void TwoWireRTC::begin()
{
  controlRegister1 = 0x00;
  controlRegister2 = CONTROL_REG2_24H_TIME_DISPLAY_SYSTEM | CONTROL_REG2_32KHZ_CLOCK_OUTPUT_DISABLE;
  
  writeRegister(I2C_RTC_ADDRESS, REGISTER_CONTROL_REGISTER_1_ADDRESS, &controlRegister1, 1);
  writeRegister(I2C_RTC_ADDRESS, REGISTER_CONTROL_REGISTER_2_ADDRESS, &controlRegister2, 1);
}

/*---------------------------------------------------------
Name:           checkIfOscillationWasHalted

Beschreibung:   prft, ob die Oszillation angehalten wurde

                Hinweis:
                Die Funktion muss vor begin aufgerufen werden,
                da bei der Initialisierung des RTC-Chips automatisch
                das Bit fr die angehaltene Oszillation gelscht wird.
 
Eingnge:       unsigned char *oscillationWasHalted
                  oscillationWasHalted zeigt nach Durchlauf
                  der Funktion an, ob die Oszillation angehalten
                  wurde.

                  oscillationWasHalted == 0: Oszillation 
                  wurde nicht angehalten

                  oscillationWasHalted > 0: Oszillation 
                  wurde angehalten

Ausgang:        unsigned char
                  Fehler bei der Abfrage der Oszillationsabfrage?

                  0: kein Fehler (Wert in oscillationWasHalted is gltig)
                  1: Fehler (Wert in oscillationWasHalted is ungltig)
---------------------------------------------------------*/
unsigned char TwoWireRTC::checkIfOscillationWasHalted(unsigned char *oscillationWasHalted)
{
  unsigned char readValue;
  unsigned char error = 0;
  
  if( readRegister(I2C_RTC_ADDRESS, REGISTER_TIME_TRIMMING_REGISTER_ADDRESS, 1, &readValue) == 1 )
  {
    *oscillationWasHalted = (readValue & 0x10);
  }
  else
  {
    error = 1;
  }
  
  return error;
}

/*---------------------------------------------------------
Name:           setTime

Beschreibung:   setzt die Uhrzeit im RTC-Chip
 
Eingnge:       DateTime *dateTime
                  Struktur mit Datum und Uhrzeit, die im RTC-Chip
                  gesetzt werden soll. 

Ausgang:        void
---------------------------------------------------------*/
void TwoWireRTC::setTime(DateTime *dateTime)
{
  unsigned char dateTimeBCD[7];
  
  dateTimeBCD[0] = decimalToBCD(dateTime->getSecond());
  dateTimeBCD[1] = decimalToBCD(dateTime->getMinute());
  dateTimeBCD[2] = decimalToBCD(dateTime->getHour());
  dateTimeBCD[3] = decimalToBCD(dateTime->getWeekday());
  dateTimeBCD[4] = decimalToBCD(dateTime->getDay());
  dateTimeBCD[5] = decimalToBCD(dateTime->getMonth());
  dateTimeBCD[6] = decimalToBCD(dateTime->getYear());
  
  writeRegister(I2C_RTC_ADDRESS, REGISTER_SECOND_COUNTER_ADDRESS, dateTimeBCD, 7);
}

/*---------------------------------------------------------
Name:           getTime

Beschreibung:   liest die Uhrzeit aus RTC-Chip
 
Eingnge:       DateTime *dateTime
                  Struktur mit Datum und Uhrzeit, in die Daten 
                  aus dem RTC-Chip gespeichert werden

Ausgang:        unsigned char
                  Fehler bei der Abfrage der Uhrzeit und
                  des Datums?

                  0: kein Fehler (dateTime enthlt die Daten aus dem RTC-Chip)
                  1: Fehler (dateTime enthlt nicht die Daten aus dem RTC-Chip)
---------------------------------------------------------*/
unsigned char TwoWireRTC::getTime(DateTime *dateTime)
{
  unsigned char dateTimeBCD[7];
  unsigned char error = 0;
  
  if( readRegister(I2C_RTC_ADDRESS, REGISTER_SECOND_COUNTER_ADDRESS, 7, dateTimeBCD) == 7 )
  {
    dateTime->setSecond(BCDToDecimal(dateTimeBCD[0]));
    dateTime->setMinute(BCDToDecimal(dateTimeBCD[1]));
    dateTime->setHour(BCDToDecimal(dateTimeBCD[2]));
    dateTime->setWeekday(BCDToDecimal(dateTimeBCD[3]));
    dateTime->setDay(BCDToDecimal(dateTimeBCD[4]));
    dateTime->setMonth(BCDToDecimal(dateTimeBCD[5]));
    dateTime->setYear(BCDToDecimal(dateTimeBCD[6]));
  }
  else
  {
    error = 1;
  }
  
  return error;
}

/*---------------------------------------------------------
Name:           setAlarmTime

Beschreibung:   setzt einen der beiden Alarmuhrzeiten im RTC-Chip
 
Eingnge:       Alarm_t alarm
                  einzustellender Alarmwert.
                  mgliche Werte: ALARM_A oder ALARM_B

                AlarmWeekdays_t weekdays
                  Wochentage fr den der Alarm gelten soll
                  mgliche Werte: 
                  SUNDAY, MONDAY, TUESDAY, WEDNESDAY, THURSDAY,
                  FRIDAY, SATURDAY, EVERY_WEEKDAY

                  die Wochentage sind ber das bitweise Oder (|)
                  verknpfbar.

                  Beispiel: Fr einen Alarm an den Wochentagen
                  Montag und Donnerstag wird der Funktion folgendes
                  bergeben:
                  (MONDAY | THURSDAY)

                  EVERY_WEEKDAY entspricht einer Oder-Verknfung aller
                  Wochentage.

                unsigned char hour
                  Stunde zu der der Alarm auslsen soll

                unsigned char minute
                  Minute zu der der Alarm auslsen soll

Ausgang:        void
---------------------------------------------------------*/
void TwoWireRTC::setAlarmTime(Alarm_t alarm, AlarmWeekdays_t weekdays, unsigned char hour, unsigned char minute)
{
  unsigned char alarmTimeBCD[] = { decimalToBCD(minute),
                                    decimalToBCD(hour),
                                    (unsigned char)weekdays };
                    
  Register_t registerAddress = REGISTER_ALARM_A_MINUTE_REGISTER_ADDRESS;
  
  if(alarm == ALARM_B)
  {
    registerAddress = REGISTER_ALARM_B_MINUTE_REGISTER_ADDRESS;
  }
  
  writeRegister(I2C_RTC_ADDRESS, registerAddress, alarmTimeBCD, 3);
}

/*---------------------------------------------------------
Name:           getAlarmTime

Beschreibung:   liest einen der beiden Alarmuhrzeiten aus dem RTC-Chip
 
Eingnge:       Alarm_t alarm
                  auszulesender Alarmwert.
                  mgliche Werte: ALARM_A oder ALARM_B

                AlarmWeekdays_t *weekdays
                  ausgelesener Wochentag an dem Alarm aktiv ist
                  (Bitkodierung siehe setAlarmTime)

                unsigned char *hour
                  ausgelesene Stunde des Alarms

                unsigned char *minute
                  ausgelesene Minute des Alarms

Ausgang:        unsigned char
                  Fehler bei der Abfrage der Alarmuhrzeit

                  0: kein Fehler (Alarmuhrzeit wurde richtig ausgelesen)
                  1: Fehler (Alarmuhrzeit wurde fehlerhaft ausgelesen)
---------------------------------------------------------*/
unsigned char TwoWireRTC::getAlarmTime(Alarm_t alarm, AlarmWeekdays_t *weekdays, unsigned char *hour, unsigned char *minute)
{
  unsigned char alarmTimeBCD[3];
  Register_t registerAddress = REGISTER_ALARM_A_MINUTE_REGISTER_ADDRESS;
  unsigned char error = 0;
  
  if(alarm == ALARM_B)
  {
    registerAddress = REGISTER_ALARM_B_MINUTE_REGISTER_ADDRESS;
  }
  
  if( readRegister(I2C_RTC_ADDRESS, registerAddress, 3, alarmTimeBCD) == 3 )
  {
    *weekdays = (AlarmWeekdays_t)alarmTimeBCD[2];
    *hour = BCDToDecimal(alarmTimeBCD[1]);
    *minute = BCDToDecimal(alarmTimeBCD[0]);
  }
  else
  {
    error = 1;
  }
  
  return error;
}

/*---------------------------------------------------------
Name:           activateAlarm

Beschreibung:   aktiviert einen der beiden Alarmuhrzeiten des RTC-Chips
 
Eingnge:       Alarm_t alarm
                  zu aktivierender Alarmwert.
                  mgliche Werte: ALARM_A oder ALARM_B

Ausgang:        void
---------------------------------------------------------*/
void TwoWireRTC::activateAlarm(Alarm_t alarm)
{
  controlRegister1 |= (alarm << 6);
  writeRegister(I2C_RTC_ADDRESS, REGISTER_CONTROL_REGISTER_1_ADDRESS, &controlRegister1, 1);
}

/*---------------------------------------------------------
Name:           deactivateAlarm

Beschreibung:   deaktiviert einen der beiden Alarmuhrzeiten des RTC-Chips
 
Eingnge:       Alarm_t alarm
                  zu deaktivierender Alarmwert.
                  mgliche Werte: ALARM_A oder ALARM_B

Ausgang:        void
---------------------------------------------------------*/
void TwoWireRTC::deactivateAlarm(Alarm_t alarm)
{
  controlRegister1 &= ~(alarm << 6);
  writeRegister(I2C_RTC_ADDRESS, REGISTER_CONTROL_REGISTER_1_ADDRESS, &controlRegister1, 1);
}

/*---------------------------------------------------------
Name:           resetAlarm

Beschreibung:   schaltet einen Alarm nach Auslsen wieder scharf
 
Eingnge:       Alarm_t alarm
                  scharf zu stellender Alarmwert.
                  mgliche Werte: ALARM_A oder ALARM_B

Ausgang:        void
---------------------------------------------------------*/
void TwoWireRTC::resetAlarm(Alarm_t alarm)
{
  controlRegister2 &= ~alarm;
  writeRegister(I2C_RTC_ADDRESS, REGISTER_CONTROL_REGISTER_2_ADDRESS, &controlRegister2, 1);
}

/*---------------------------------------------------------
Name:           setTimeTrimming

Beschreibung:   setzt den Kalibrierwert des RTC-Chips
 
Eingnge:       char newTimeTrimmingValue
                  neuer Kalibrierwert

Ausgang:        void
---------------------------------------------------------*/
void TwoWireRTC::setTimeTrimming(char newTimeTrimmingValue)
{
  unsigned char trimmingValue;
  
  if(newTimeTrimmingValue >= 0)
    trimmingValue = (unsigned char)newTimeTrimmingValue;
  else
    trimmingValue = (unsigned char)(128 + newTimeTrimmingValue);
  
  writeRegister(I2C_RTC_ADDRESS, REGISTER_TIME_TRIMMING_REGISTER_ADDRESS, &trimmingValue, 1);
}

/*---------------------------------------------------------
Name:           getTimeTrimming

Beschreibung:   liest den Kalibrierwert des RTC-Chips
 
Eingnge:       char *timeTrimmingValue
                  ausgelesener Kalibrierwert

Ausgang:        unsigned char
                  Fehler bei der Abfrage des Kalibrierwerts

                  0: kein Fehler (Kalibrierwert wurde richtig ausgelesen)
                  1: Fehler (Kalibrierwert wurde fehlerhaft ausgelesen)
---------------------------------------------------------*/
unsigned char TwoWireRTC::getTimeTrimming(char *timeTrimmingValue)
{
  unsigned char readValue;
  unsigned char error = 0;
  
  if( readRegister(I2C_RTC_ADDRESS, REGISTER_TIME_TRIMMING_REGISTER_ADDRESS, 1, &readValue) == 1 )
  {
    if((readValue & 0x40) == 0)
      *timeTrimmingValue = (char)readValue;
    else
      *timeTrimmingValue = ((char)readValue) - 128;
  }
  else
  {
    error = 1;
  }
  
  return error;
}

/*---------------------------------------------------------
Name:           setPeriodicInterrupt

Beschreibung:   stellt den periodischen Interruptbetrieb
                des RTC-Chips ein, der am Ausgang INTRA
                ausgegeben wird
 
Eingnge:       PeriodicInterruptSelection_t periodicInterruptSelection
                  Art des periodischen Betriebs:

                  PERIODIC_INTERRUPT_OFF
                    kein periodischer Interrupt (INTRA-Leitung liegt auf HIGH)

                  PERIODIC_INTERRUPT_FIXED_AT_LOW
                    kein periodischer Interrupt (INTRA-Leitung liegt auf LOW)

                  PERIODIC_INTERRUPT_PULSE_MODE_2_HZ
                    Pulsmodus mit 2 Hz

                  PERIODIC_INTERRUPT_PULSE_MODE_1_HZ
                    Pulsmodus mit 1 Hz

                  PERIODIC_INTERRUPT_LEVEL_MODE_EVERY_SECOND
                    Interrupt jede Sekunde (synchron mit internem RTC-Sekundenzhler)

                  PERIODIC_INTERRUPT_LEVEL_MODE_EVERY_MINUTE
                    Interrupt jede Minute (bei Sekunde 0)

                  PERIODIC_INTERRUPT_LEVEL_MODE_EVERY_HOUR
                    Interrupt jede Stunde (bei Minute und Sekunde 0)

                  PERIODIC_INTERRUPT_LEVEL_MODE_EVERY_MONTH
                    Interrupt jeden Monat (jeweils am 1. um 0:00:00)

                  Hinweis:
                  Die Interrupts mit der Bezeichnung LEVEL_MODE werden
                  nicht automatisch zurckgesetzt, sondern mssen durch
                  den Aufruf der Methode resetPeriodicInterrupt wieder
                  scharf geschaltet werden.

Ausgang:        void
---------------------------------------------------------*/
void TwoWireRTC::setPeriodicInterrupt(PeriodicInterruptSelection_t periodicInterruptSelection)
{
  // die unteren drei Bits lschen, da hier die Daten fr den periodischen Interrupt neu
  // geschrieben werden.
  controlRegister1 &= ~0x07; 
  controlRegister1 |= periodicInterruptSelection;
  writeRegister(I2C_RTC_ADDRESS, REGISTER_CONTROL_REGISTER_1_ADDRESS, &controlRegister1, 1);
}

/*---------------------------------------------------------
Name:           resetPeriodicInterrupt

Beschreibung:   schaltet den periodischen Interrupt bei
                Auswahl eines der Level-Modi wieder scharf
                (siehe setPeriodicInterrupt)
 
Eingnge:       void

Ausgang:        void
---------------------------------------------------------*/
void TwoWireRTC::resetPeriodicInterrupt(void)
{
  controlRegister2 &= ~(1<<2);
  writeRegister(I2C_RTC_ADDRESS, REGISTER_CONTROL_REGISTER_2_ADDRESS, &controlRegister2, 1);
}

/**********************************************************
private Methoden
**********************************************************/

/*---------------------------------------------------------
Name:           writeRegister

Beschreibung:   schreibt eine vorgegebene Anzahl Register des
                des RTC-Chips
 
Eingnge:       unsigned char address
                  I2C-Adresse des RTC-Chips

                unsigned char regAddress
                  Registeradresse ab der die Daten geschrieben
                  werden sollen

                unsigned char *data
                  zu schreibende Daten

                unsigned char datalength
                  Anzahl der zu schreibenden Daten

Ausgang:        void
---------------------------------------------------------*/
void TwoWireRTC::writeRegister(unsigned char address, Register_t regAddress, unsigned char *data, unsigned char datalength)
{
  char temp_data[datalength+1];
  temp_data[0] = regAddress << 4;
  for(unsigned char i = 0; i < datalength; i++)
  {
    temp_data[i+1] = *(data+i);
  }
  Wire.beginTransmission(address);
  // Wire.write(regAddress << 4);
  // Wire.write(data, datalength);
  Wire.write(&temp_data[0], (datalength+1));
  Wire.endTransmission();
}

/*---------------------------------------------------------
Name:           readRegister

Beschreibung:   schreibt eine vorgegebene Anzahl Register des
                des RTC-Chips
 
Eingnge:       unsigned char address
                  I2C-Adresse des RTC-Chips

                unsigned char regAddress
                  Registeradresse ab der die Daten gelesen
                  werden sollen

                unsigned char length
                  Anzahl der zu lesenen Daten

                unsigned char *data
                  gelesene Daten

Ausgang:        unsigned char
                  Anzahl der gelesenen Bytes
                  (kann von der vorgegebenen Anzahl abweichen
                  -> Fehler)
---------------------------------------------------------*/
unsigned char TwoWireRTC::readRegister(unsigned char address, Register_t regAddress, unsigned char length, unsigned char *data)
{
  unsigned char readBytes = 0;
  unsigned char actIndex = 0;
  unsigned char temp = 0;
  
  Wire.requestFrom((int)address, length+1);
  
  // erstes empfangenes Byte verwerfen (Wert Adresse 0x0F)
  if(Wire.available())
    Wire.read();
    
  while(/*Wire.available() && */(length != 0))
  {
    temp = Wire.read();
    if(actIndex >= regAddress)
    {
      data[readBytes] = temp;
      readBytes++;
      length--;
    }
    
    actIndex++;
  }
  
  return readBytes;
}

/*---------------------------------------------------------
Name:           decimalToBCD

Beschreibung:   rechnet Dezimalwerte in das BCD-System um
                (der RTC-Chip arbeitet intern mit BCD-Daten)
 
Eingnge:       unsigned char decimal
                  umzurechnender Dezimalwert

Ausgang:        unsigned char
                  BCD-Wert der bergebenen Dezimalzahl
---------------------------------------------------------*/
unsigned char TwoWireRTC::decimalToBCD(unsigned char decimal)
{
  return (((decimal / 10) << 4) | (decimal % 10));
}

/*---------------------------------------------------------
Name:           BCDToDecimal

Beschreibung:   rechnet BCD-Werte in das Dezimal-System um
                (der RTC-Chip arbeitet intern mit BCD-Daten)
 
Eingnge:       unsigned char bcd
                  umzurechnender BCD-Wert

Ausgang:        unsigned char
                  Dezimalwert der bergebenen BCD-Zahl
---------------------------------------------------------*/
unsigned char TwoWireRTC::BCDToDecimal(unsigned char bcd)
{
  return (((((bcd)>>4) & 0x0F) * 10) + ((bcd) & 0x0F));
}



/**********************************************************
Vorinstantiiertes Objekt
**********************************************************/
TwoWireRTC RTC = TwoWireRTC();